/*******************************************************************************
 * @license
 * Copyright (c) 2009 - 2013 IBM Corporation, VMware and others.
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License v1.0
 * (http://www.eclipse.org/legal/epl-v10.html), and the Eclipse Distribution
 * License v1.0 (http://www.eclipse.org/org/documents/edl-v10.html).
 *
 * Contributors:
 *  IBM Corporation - initial API and implementation
 *     Andy Clement - hacked to support our needs in scripted (needs more work)
 ******************************************************************************/

/*global define window uri alert $ */
/*jslint regexp:false browser:true forin:true*/

define(['require', 'scripted/navigator/explorer', "jquery", "scripted/utils/pageState", "scripted/utils/navHistory", "scriptedLogger"],
	function(require, mExplorer, mJquery, mPageState, mNavHistory, scriptedLogger) {

	/**
	 * Tree model used by the FileExplorer
	 * TODO: Consolidate with eclipse.TreeModel.
	 */

	function Model(serviceRegistry, root, fileClient, treeId) {
		this.registry = serviceRegistry;
		this.root = root;
		this.fileClient = fileClient;
		this.treeId = treeId;
	}
	Model.prototype = new mExplorer.ExplorerModel();
	Model.prototype.getRoot = function(onItem) {
		onItem(this.root);
	};

	function processNavigatorParent(parent, children) {
		//link the parent and children together
		parent.children = children;
		for (var e in children) {
			var child = children[e];
			child.parent = parent;
		}
	}


	Model.prototype.getChildren = function( /* dojo.data.Item */ parentItem, /* function(items) */ onComplete) {
		// the parent already has the children fetched
		if (parentItem.children) {
			onComplete(parentItem.children);
		} else if (parentItem.Directory !== undefined && parentItem.Directory === false) {
			onComplete([]);
		} else if (parentItem.Location) {
			if (parentItem.ChildrenLocation) {
				var xhrobj = new XMLHttpRequest();
				try {
					var url = '/fs_list/' + parentItem.ChildrenLocation;
//					scriptedLogger.debug("url is " + url, "EXPLORER_TABLE");
					xhrobj.open("GET", url, true);
					xhrobj.onreadystatechange = function() {
						if (xhrobj.readyState === 4) {
							// TODO status =200 should proceed otherwise error (xhrObj.status     ==200)
							var children = JSON.parse(xhrobj.responseText).children;
							// skip kids that start '.'
							if (children) {
								var newchildren = [];
								// TODO move to serverside and make it consistent with search/lookfor - see filesystem.js.ignore()
								for (var i=0;i<children.length;i++) {
									var kid = children[i];
									if (kid.name[0]!=='.' || !(kid.name==='.git' || kid.name==='.svn')) {
										newchildren.push(kid);
									}
								}
								children = newchildren;
								children = children.sort(function (a,b) {
									if (a.directory) {
										if (b.directory) {
											return (a.name.toLowerCase().localeCompare(b.name.toLowerCase()));
										} else {
											return -1;
										}
									} else {
										if (b.directory) {
											return +1;
										} else {
											return (a.name.toLowerCase().localeCompare(b.name.toLowerCase()));
										}
									}
									// return (a.name.toLowerCase().localeCompare(b.name.toLowerCase()));
								});
//								children = children.sort(function(a,b) {
//									 return (a.name.toLowerCase().localeCompare(b.name.toLowerCase()));
//								});
							}
							processNavigatorParent(parentItem, children);
							onComplete(children);
						}
					};
					xhrobj.send();
				} catch (e) {
					scriptedLogger.error("xhr failed " + e, "EXPLORER_TABLE");
				}
			}

		} else {
			onComplete([]);
		}
	};
	Model.prototype.constructor = Model;

	/**
	 * Renders json items into columns in the tree
	 */

	function FileRenderer(options, explorer, commandService, contentTypeService) {
		this.explorer = explorer;
		this.commandService = commandService;
		this.contentTypeService = contentTypeService;
		this.openWithCommands = null;
		this.actionScopeId = "fileFolderCommands";
		this._init(options);
		this.target = "_self";
	}
	FileRenderer.prototype = new mExplorer.SelectionRenderer();

	// we are really only using the header for a spacer at this point.
	FileRenderer.prototype.getCellHeaderElement = function(col_no) {
		switch (col_no) {
		case 0:
		case 1:
		case 2:
			return $('<th/>', {
				style: "height: 8px;"
			})[0];
		}
	};

	//This is an optional function for explorerNavHandler. It provides the div with the "href" attribute.
	//The explorerNavHandler hooked up by the explorer will check if the href exist as the attribute and react on enter key press.
	FileRenderer.prototype.getRowActionElement = function(tableRowId) {
		return document.getElementById(tableRowId + "NameColumn");
	};

	FileRenderer.prototype.onRowIterate = function(model) {
		if (this.explorer.navHandler) {
			this.explorer.navHandler.cursorOn(model);
		}
	};

	FileRenderer.prototype.setTarget = function(target) {
		this.target = target;

		$('.targetSelector').each(function(index, node) {
			node.target = target;
		});
	};

	FileRenderer.prototype.getCellElement = function(col_no, item, tableRow) {
		function isImage(contentType) {
			switch (contentType && contentType.id) {
			case "image.jpeg":
			case "image.png":
			case "image.gif":
			case "image.ico":
			case "image.tiff":
			case "image.svg":
				return true;
			}
			return false;
		}

		function addImageToLink(contentType, link) {
			switch (contentType && contentType.id) {
			case "image.jpeg":
			case "image.png":
			case "image.gif":
			case "image.ico":
			case "image.tiff":
			case "image.svg":
				var thumbnail = $('<img/>', {
					src: item.Location
				});
				$(link).append(thumbnail);

				$(thumbnail).addClass("thumbnail");
				break;
			default:
				if (contentType && contentType.image) {
					var image = $('<img/>', {
						src: contentType.image
					});
					$(link).append(image);
					// to minimize the height/width in case of a large one
					$(image).addClass("thumbnail");
				} else {
					var fileIcon = $('<span/>');
					$(link).append(fileIcon);

					$(fileIcon).addClass("core-sprite-file_model modelDecorationSprite");
				}
			}
		}

		switch (col_no) {

		case 0:
			var col = document.createElement('td');

			$(col).empty();
			var span = $('<span/>', {
				id: tableRow.id + "Actions"
			});
			$(col).append(span);
			var link;
			//			scriptedLogger.debug("name is '"+item.name+"'", "EXPLORER_TABLE");
			if (item.directory) {
				this.getExpandImage(tableRow, span);
			} else if (item.name === "") {
				var fileIcon1 = $('<span/>');
				$(span).append(fileIcon1);
				$(fileIcon1).addClass("core-sprite-blank_model modelDecorationSprite2");
			} else {
				var fileIcon = $('<span/>');
				$(span).append(fileIcon);
				$(fileIcon).addClass("core-sprite-blank_model modelDecorationSprite2");
			}
			var span2;
			var path = mPageState.generateUrl(item.Location);
			if (item.directory) {
				span2 = $('<span/>');
				$(span).append(span2);

				// When the directory name is clicked, simulate a keypress on the expand/collapse image
				tableRow.onclick = function(event){
					$(event.currentTarget).find('.modelDecorationSprite').click();
				};

			} else {
				span2 = $('<a/>', {
					href: path
					});
				$(span).append(span2);
			}

			var textnode = document.createTextNode(item.name);
			$(span2).append(textnode);

			if (!item.directory) {
				$(span2).click(mNavHistory.handleNavigationEvent);
			}

			return col;
		}
	};
	FileRenderer.prototype.constructor = FileRenderer;

	/**
	 * Creates a new file explorer.
	 * @name orion.explorer-table.FileExplorer
	 * @class A user interface component that displays a table-oriented file explorer
	 * @param {orion.serviceRegistry.ServiceRegistry} options.serviceRegistry
	 * @param {Object} options.treeRoot
	 * @param {orion.selection.Selection} options.selection
	 * @param {orion.searchClient.Searcher} options.searcher
	 * @param {orion.fileClient.FileClient} options.fileClient
	 * @param {orion.commands.CommandService} options.commandService
	 * @param {orion.core.ContentTypeService} options.contentTypeService
	 * @param {String} options.parentId
	 * @param {String} options.breadcrumbId
	 * @param {String} options.toolbarId
	 * @param {String} options.selectionToolsId
	 */

	function FileExplorer(options) {
		this.registry = options.serviceRegistry;
		this.treeRoot = options.treeRoot;
		this.selection = options.selection;
		this.searcher = options.searcher;
		this.fileClient = options.fileClient;
		this.commandService = options.commandService;
		this.contentTypeService = options.contentTypeService;
		this.parentId = options.parentId;
		this.breadcrumbId = options.breadcrumbId;
		this.toolbarId = options.toolbarId;
		this.selectionToolsId = options.selectionToolsId;
		this.model = null;
		this.myTree = null;
		this.renderer = new FileRenderer({
			checkbox: true,
			decorateAlternatingLines: true,
			cachePrefix: "Navigator"
		}, this, this.commandService, this.contentTypeService);
		this.preferences = options.preferences;
		this.setTarget();
		// TODO ASC made conditional
		if (this.preferences) {
			this.storageKey = this.preferences.listenForChangedSettings($.proxy(onStorage, this));
		}

		// highlight the current main editor in explorer when main editor is created
		var self = this;
		$(document).on('paneCreated', function(e, pane) {
			if (self.model && pane.isMain && pane.paneId === 'scripted.editor') {
				self.highlight(pane.editor.getFilePath());
			}
		});
	}

	FileExplorer.prototype = new mExplorer.Explorer();

	// we have changed an item on the server at the specified parent node
	FileExplorer.prototype.changedItem = function(parent) {
		var self = this;
		this.fileClient.fetchChildren(parent.ChildrenLocation).then(function(children) {
//			mUtil.processNavigatorParent(parent, children);
			//If a key board navigator is hooked up, we need to sync up the model
			if (self.navHandler) {
				self.navHandler.refreshModel(self.model);
			}

			$.proxy(self.myTree.refreshAndExpand, self.myTree)(parent, children);
		});
	};

	FileExplorer.prototype.getNameNode = function(item) {
		var rowId = this.model.getId(item);
		if (rowId) {
			// I know this from my renderer below.
			return document.getElementById(rowId + "NameColumn");
		}
	};

	//This is an optional function for explorerNavHandler. It changes the href of the window.locatino to navigate to the parent page.
	//The explorerNavHandler hooked up by the explorer will check if this optional function exist and call it when left arrow key hits on a top level item that is aleady collapsed.
	FileExplorer.prototype.scopeUp = function() {
		if (this.treeRoot && this.treeRoot.Parents) {
			if (this.treeRoot.Parents.length === 0) {
				window.location.href = "#";
			} else if (this.treeRoot.Parents[0].ChildrenLocation) {
				window.location.href = "#" + this.treeRoot.Parents[0].ChildrenLocation;
			}
		}
	};

	FileExplorer.prototype.setTarget = function() {

		var preferences = this.preferences;
		var renderer = this.renderer;

		// TODO ASC need default for target?
		if (preferences) {
			this.preferences.getPreferences('/settings', 2).then(function(prefs) {

				var storage = JSON.parse(prefs.get("General"));

				if (storage) {
					var target = preferences.getSetting(storage, "Navigation", "Links");

					if (target === "Open in new tab") {
						target = "_blank";
					} else {
						target = "_self";
					}

					renderer.setTarget(target);
				}
			});
		}
	};

	FileExplorer.prototype.onStorage = function(e) {
		if (e.key === this.storageKey) {
			this.setTarget();
		}
	};


	/**
     * Expand the tree if necessary and highlight the specific file. By default it will expand an directory selection, unless explicitly set to false. By default, it will also scroll
     * any elements that are highlighted but not visible in the navigator, unless skipping scrolling is explicitly set to true.
     */
	FileExplorer.prototype.highlight = function( /*String*/ fileintree, expand, skipScrolling) {
       // if expand is not defined, expand by default
	   if (expand === undefined) {
	      expand = true;
	   }

	   if (skipScrolling === undefined) {
			skipScrolling = false;
	   }

		/*Remove any existing highlights*/
		var element = $('.highlightrow')[0];
		if (element){
			var index = $('.highlightrow').index();
			if (index % 2 === 1){
				$('.highlightrow').addClass('darkTreeTableRow');
			} else {
				$('.highlightrow').addClass('lightTreeTableRow');
			}
			if (element.childNodes) {
				$(element.childNodes[1]).removeClass("secondaryColumnDark");
				$(element.childNodes[1]).addClass("secondaryColumn");
			}
			$('.highlightrow').removeClass('highlightrow');
		}

		var self = this;
		var id = this.model.getIdFromString(fileintree);
		this._highlightingId = id;

		element = document.getElementById(id);
		if (element === null || element === undefined) {
			var expandSection = function(root, splits, index) {
				if (index < (splits.length - 1)) {
					var newroot = root + "/" + splits[index];
					var stringid = self.model.getIdFromString(newroot);
					self.renderer.expand(stringid, function() {
						expandSection(newroot, splits, index + 1);
					});
				} else {
					var stringid = self.model.getIdFromString(root + "/" + splits[index]);
					var element = document.getElementById(id);
					if (element) {
						$(element).addClass("highlightrow");
						$(element).removeClass("lightTreeTableRow");
						$(element).removeClass("darkTreeTableRow");
						if (element.childNodes) {
							$(element.childNodes[1]).removeClass("secondaryColumn");
							$(element.childNodes[1]).addClass("secondaryColumnDark");
						}
						self.renderer.expand(stringid);
					}
				}
			};
			// need to open the parents
			var root = window.fsroot;
			// root = /foo/bar
			// fileintree = /foo/bar/here/there/file.js
			var topmostparent = fileintree.substr(root.length + 1); // topmostparent = here/there/file.js
			var splits = topmostparent.split('/');

			if (expand === true) {
			    expandSection(root, splits, 0);
			}

		} else {
			$(element).addClass("highlightrow");
			$(element).removeClass("lightTreeTableRow");
			$(element).removeClass("darkTreeTableRow");
			$(element.childNodes[1]).removeClass("secondaryColumn");
			$(element.childNodes[1]).addClass("secondaryColumnDark");

			// Scroll if not visible
			if (skipScrolling === false) {
				var currentPosition = $(element).offset();

				var navigatorHeight = $("#navigator-container").height();

				if (navigatorHeight < currentPosition.top) {
					$("#navigator-container").scrollTop(currentPosition.top);
				}
			}


			if (expand === true) {
                 this.renderer.expand(id);
			}
		}
	};

	/**
	 * Load the resource at the given path.
	 * @param path The path of the resource to load
	 * @param [force] If true, force reload even if the path is unchanged. Useful
	 * when the client knows the resource underlying the current path has changed.
	 * @param postLoad a function to call after loading the resource
	 */
	FileExplorer.prototype.loadResourceList = function(path, force, postLoad) {
		// scriptedLogger.info("loadResourceList old " + this._lastHash + " new " + path, "EXPLORER_TABLE");
		// TODO ASC absolute paths... for me
		// path = mUtil.makeRelative(path);
		if (!force && path === this._lastHash) {
			return;
		}

		this._lastHash = path;
		var parent = document.getElementById(this.parentId);

		// we are refetching everything so clean up the root
		this.treeRoot = {};

		if (force || (path !== this.treeRoot.Path)) {
			//the tree root object has changed so we need to load the new one

			// Progress indicator
			var progress = document.getElementById("progress");
			if (!progress) {
				$(parent).empty();
				progress = $('<div/>', {
					id: "progress"
				});
				$(parent).append(progress);
			}
			$(progress).empty();

			var progressTimeout = setTimeout(function() {
				$(progress).empty();

				var b = $('<b/>');

				$(progress).append(document.createTextNode("Loading "));
				$(b).append(document.createTextNode(path));

				$(progress).append(b);
				$(progress).append(document.createTextNode("..."));

			}, 500); // wait 500ms before displaying

			this.treeRoot.Path = path;
			var self = this;
			this.postLoad = postLoad;

			// fileClient.js:156
/*
{
	Directory: true,
	Length: 0,
	LocalTimeStamp: 0,
	Name: "File Servers",
	Location: "/",
	Children: _fileSystemsRoots,
	ChildrenLocation: "/"
}
*/

			//this.treeRoot = [{Directory:true,Length:0,LocalTimeStamp:0,Name:"File Servers",Location:path,}];
			var place = this;

			var xhrobj = new XMLHttpRequest();
			try {
				var url = '/fs_list/' + path;
//				scriptedLogger.debug("url is " + url, "EXPLORER_TABLE");
				xhrobj.open("GET", url, true);
				xhrobj.onreadystatechange = function() {
					if (xhrobj.readyState === 4) {
						var response = JSON.parse(xhrobj.responseText);
						// skip kids that start '.'
						if (response.children) {
							var newchildren = [];
							for (var i=0;i<response.children.length;i++) {
								var kid = response.children[i];
								// TODO this is duplicate code
								if (kid.name[0]!=='.' || !(kid.name==='.git' || kid.name==='.svn')) {
									newchildren.push(kid);
								}
//								if (kid.name.lastIndexOf('.',0)!==0) {
//									newchildren.push(kid);
//								}
							}
							response.children = newchildren;
							response.children = response.children.sort(function (a,b) {
								if (a.directory) {
									if (b.directory) {
										return (a.name.toLowerCase().localeCompare(b.name.toLowerCase()));
									} else {
										return -1;
									}
								} else {
									if (b.directory) {
										return +1;
									} else {
										return (a.name.toLowerCase().localeCompare(b.name.toLowerCase()));
									}
								}
								// return (a.name.toLowerCase().localeCompare(b.name.toLowerCase()));
							});

//							response.children = response.children.sort(function (a,b) {
//								return (a.name.toLowerCase().localeCompare(b.name.toLowerCase()));
//							});
						}

						place.treeRoot = response;
						place.model = new Model(place.registry, place.treeRoot, null);
						place.createTree(place.parentId, place.model, {
							onCollapse: function(model) {
								if (self.navHandler) {
									self.navHandler.onCollapse(model);
								}
							}
						}, function() {
							if (typeof self.postLoad === "function") {
								self.postLoad();
							}
						});
					}
				};
				xhrobj.send();
			} catch (e) {
				scriptedLogger.error("xhr failed " + e, "EXPLORER_TABLE");
			}
		}
	};

	FileExplorer.prototype.fullRefresh = function(postRefresh) {
		this.loadResourceList(window.fsroot/*pageParams.resource*/, true, postRefresh);
	};
	/**
	 * Clients can connect to this function to receive notification when the root item changes.
	 * @param {Object} item
	 */
	FileExplorer.prototype.onchange = function(item) {};
	FileExplorer.prototype.constructor = FileExplorer;

	//return module exports
	return FileExplorer;
});
